from structio.abc import SignalManager
from structio.signals import set_signal_handler
from structio.core.run import current_loop
from types import FrameType
import warnings
import signal

from structio.util.ki import currently_protected


class SigIntManager(SignalManager):
    """
    Handles Ctrl+C
    """

    def __init__(self):
        self.installed = False

    @staticmethod
    async def _handle(sig: int, frame: FrameType):
        if currently_protected():
            current_loop().signal_notify(sig, frame)
        else:
            current_loop().raise_ki()

    def install(self):
        if signal.getsignal(signal.SIGINT) != signal.default_int_handler:
            warnings.warn(
                f"structio has detected a custom SIGINT handler and won't touch it: keep in mind"
                f" this is likely to break KeyboardInterrupt delivery!"
            )
            return
        set_signal_handler(signal.SIGINT, self._handle)
        self.installed = True

    def uninstall(self):
        if self.installed:
            signal.signal(signal.SIGINT, signal.default_int_handler)
            self.installed = False
