from structio.abc import Debugger
from structio.core.task import Task


class SimpleDebugger(Debugger):
    """
    A simple debugger for structio
    """

    def on_start(self):
        print("## Started running")

    def on_exit(self):
        print("## Finished running")

    def on_task_spawn(self, task: Task):
        print(f">> A task named '{task.name}' was spawned")

    def on_task_exit(self, task: Task):
        print(f"<< Task '{task.name}' exited")

    def before_task_step(self, task: Task):
        print(f"-> About to run a step for '{task.name}'")

    def after_task_step(self, task: Task):
        print(f"<- Ran a step for '{task.name}'")

    def before_sleep(self, task: Task, seconds):
        print(f"# About to put '{task.name}' to sleep for {seconds:.2f} seconds")

    def after_sleep(self, task: Task, seconds):
        print(f"# Task '{task.name}' slept for {seconds:.2f} seconds")

    def before_io(self, timeout):
        if timeout is None:
            timeout = float("inf")
        print(f"!! About to check for I/O for up to {timeout:.2f} seconds")

    def after_io(self, timeout):
        print(f"!! Done I/O check (waited for {timeout:.2f} seconds)")

    def before_cancel(self, task: Task):
        print(f"// About to cancel '{task.name}'")

    def after_cancel(self, task: Task):
        print(f"// Cancelled '{task.name}'")

    def on_exception_raised(self, task: Task, exc):
        print(f"== '{task.name}' raised {repr(exc)}")

    def on_io_schedule(self, resource, event: str):
        print(f"|| Resource {resource!r} was scheduled for {event}")

    def on_io_unschedule(self, resource):
        print(f"|| Resource {resource!r} was unscheduled")
